#include <math.h>
#include <qimage.h> 

#include "pixel.h"
#include "globals.h"

void compute_diff( Pixel* p )
{
	p->d_r = float( fabs( p->f_r - p->b_r ) );
	p->d_g = float( fabs( p->f_g - p->b_g ) );
	p->d_b = float( fabs( p->f_b - p->b_b ) );
}

void consume_diff( Pixel* p, float d, char color )
{
	float *f_p, *b_p, *d_p;
	switch( color )
	{
	case 'r':
		f_p = &p->f_r;
		d_p = &p->d_r;
		b_p = &p->b_r;
		break;
	case 'g':
		f_p = &p->f_g;
		d_p = &p->d_g;
		b_p = &p->b_g;
		break;
	default:
	case 'b':
		f_p = &p->f_b;
		d_p = &p->d_b;
		b_p = &p->b_b;
		break;
	}
	if ( d >= *d_p )
	{
		*f_p = *b_p;
		*d_p = 0.0;
	}
	else if ( *f_p < *b_p )
	{
		*f_p += d;
		compute_diff( p );
	}
	else if ( *f_p > *b_p )
	{
		*f_p -= d;
		compute_diff( p );
	}	
}

int image_width( char* file )
{
	QImage image( file );
	if ( image.isNull() ) return 0;
	return image.width();
}

int image_height( char* file )
{
	QImage image( file );
	if ( image.isNull() ) return 0;
	return image.height();	
}

//returns "diff amount" as a sum of diffs over entire image
float load_pixels( Pixel** p, char* file, int width, int height )
{
	if ( !file ) return load_random( p, width, height );

	QImage image( file );
	if ( image.isNull() ) return load_random( p, width, height );

	image = image.mirror(); 

	float d_t = 0.0;

	for ( int x = 0 ; x < width ; x++ )
	{
		for ( int y = 0 ; y < height ; y++ )
		{
			if ( x >= image.width() || y >= image.height() )
			{
				p[x][y].b_r = 0.0;
				p[x][y].b_g = 0.0;
				p[x][y].b_b = 0.0;
			}
			else
			{
				p[x][y].b_r = qRed  ( image.pixel( x, y ) ) / 255.0 ;
				p[x][y].b_g = qGreen( image.pixel( x, y ) ) / 255.0 ;
				p[x][y].b_b = qBlue ( image.pixel( x, y ) ) / 255.0 ;
			}
			compute_diff( & p[x][y] );
			d_t += p[x][y].d_r + p[x][y].d_g + p[x][y].d_b;
		}
	}
	return d_t;
}

//returns "diff amount" as a sum of diffs over entire image
float load_random( Pixel** p, int width, int height )
{
	float d_t = 0.0;

	float r = RandFloat();
	float g = RandFloat();
	float b = RandFloat();

	for ( int x = 0 ; x < width ; x++ )
	{
		for ( int y = 0 ; y < height ; y++ )
		{
			p[x][y].b_r = r;
			p[x][y].b_g = g;
			p[x][y].b_b = b;
			compute_diff( & p[x][y] );
			d_t += p[x][y].d_r + p[x][y].d_g + p[x][y].d_b;
		}
	}
	return d_t;
}

void save_pixels( Pixel** p, char* file, int width, int height )
{
	QImage image( width, height, 32 );
	for ( int x = 0 ; x < width ; x++ )
	{
		for ( int y = 0 ; y < height ; y++ )
		{
			image.setPixel
				( 
				x, y, qRgb
				 ( p[x][y].f_r * 255, p[x][y].f_g * 255, p[x][y].f_b * 255 ) 
				);
		}
	}
	image.mirror().save( file, "PNG" );
}

